package com.xhkjedu.smarking.service.report.reportother;

import com.alibaba.fastjson.JSON;
import com.xhkjedu.smarking.mapper.exam.*;
import com.xhkjedu.smarking.mapper.report.reportclass.*;
import com.xhkjedu.smarking.mapper.report.reportother.MsrExamMapper;
import com.xhkjedu.smarking.mapper.report.reportother.MsrReportparamMapper;
import com.xhkjedu.smarking.mapper.report.reportstu.MsrStudentMapper;
import com.xhkjedu.smarking.mapper.report.reportsubject.MsrSubjectClassMapper;
import com.xhkjedu.smarking.mapper.report.reportsubject.MsrSubjectMapper;
import com.xhkjedu.smarking.mapper.stupaper.MsPaperStudentMapper;
import com.xhkjedu.smarking.model.exam.*;
import com.xhkjedu.smarking.model.paper.MsPaper;
import com.xhkjedu.smarking.model.report.reportclass.*;
import com.xhkjedu.smarking.model.report.reportother.MsrExam;
import com.xhkjedu.smarking.model.report.reportother.MsrReportparam;
import com.xhkjedu.smarking.model.report.reportstu.MsrStudent;
import com.xhkjedu.smarking.model.report.reportsubject.MsrSubject;
import com.xhkjedu.smarking.model.report.reportsubject.MsrSubjectClass;
import com.xhkjedu.smarking.utils.MarkingUtil;
import com.xhkjedu.smarking.vo.report.reportclass.ClassTopVo;
import com.xhkjedu.smarking.vo.report.reportother.*;
import com.xhkjedu.utils.N_Utils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description 阅卷报告-总体分析表 服务实现类
 * @Author auto
 * @Date 2024-12-09
 */
@Service
public class MsrExamService {
    @Resource
    private MsrExamMapper msrExamMapper;
    @Resource
    private MsExamMapper examMapper;
    @Resource
    private MsSubjectMapper subjectMapper;
    @Resource
    private MsClassMapper classMapper;
    @Resource
    private MsClassStudentMapper classStudentMapper;
    @Resource
    private MsPaperStudentMapper paperStudentMapper;
    @Resource
    private MsrSubjectMapper msrSubjectMapper;
    @Resource
    private MsrStudentMapper msrStudentMapper;
    @Resource
    private MsrReportparamMapper msrReportparamMapper;
    @Resource
    private MsrSubjectClassMapper msrSubjectClassMapper;
    @Resource
    private MsrClassSubjectMapper msrClassSubjectMapper;
    @Resource
    private MsrClassTopMapper msrClassTopMapper;
    @Resource
    private MsrClassSubjectGradeMapper msrClassSubjectGradeMapper;


    /**
     * @Description 生成报告
     * @Date 2024/12/9 15:43
     * @Author YWX
     * @Param [examid]
     * @Return void
     **/
    public void generateExamReport(Integer examid) {
        MsExam exam = examMapper.selectByPrimaryKey(examid);
        Integer exammode = exam.getExammode();
        List<MsPaper> subjects = subjectMapper.listReportSubject(examid, exammode);//考试报告科目列表
        if (N_Utils.isListEmpty(subjects)) return;
        List<MsClass> classes = classMapper.listClassByExamId(examid);//获取考试关联班级
        List<MsClassStudent> students = classStudentMapper.listStudentByExamId(examid);//获取考试关联学生
        List<Integer> mpids = new ArrayList<>();
        for (MsPaper subject : subjects) {
            Integer mpid = subject.getMpid();
            mpids.add(mpid);
        }
        List<MsrStudent> paperStudents = paperStudentMapper.listPaperStudentByExamId(examid, mpids);//试学生成绩列表
        for (MsrStudent student : paperStudents) {
            student.setScorerate(N_Utils.getDoubleDivideAndMulitiply(student.getStuscore(), student.getFullscore()));
        }

        //等级参数
        List<MsrReportparam> djParams = msrReportparamMapper.listReportparamByRpbelong(examid, "subjectshow", "all");
        List<SzJsonVo> djList = new ArrayList<>();
        if (N_Utils.isListNotEmpty(djParams)) {
            djList = JSON.parseArray(djParams.get(0).getSzjson(), SzJsonVo.class);
        }

        Integer classnum = N_Utils.obj2Int(classes.stream().map(MsClass::getClassid).distinct().count());
        Integer totalnum = N_Utils.obj2Int(students.stream().map(MsClassStudent::getStudentid).distinct().count());
        Integer stunum = N_Utils.obj2Int(paperStudents.stream().filter(s -> s.getSstate().equals(2)).map(MsrStudent::getStudentid).distinct().count());
        Integer missnum = totalnum - stunum;
        double fullscore = subjects.stream().mapToDouble(MsPaper::getPscore).sum();
        List<MsrStudent> schoolStudents = new ArrayList<>();//学校学生成绩
        List<MsrStudent> classStudents = new ArrayList<>();//班级学生成绩(总分)
        List<MsrStudent> classSubjectStudents = new ArrayList<>();//班级科目学生成绩
        Map<String, StuRankVo> schoolStuRank = new HashMap<>();//学校学生分数排名
        Map<String, StuRankVo> schoolSubStuRank = new HashMap<>();//学校科目学生分数排名
        Map<String, StuRankVo> classStuRank = new HashMap<>();//班级学生分数排名
        Map<String, StuRankVo> classSubStuRank = new HashMap<>();//班级科目学生分数排名
        Map<Integer, List<MsrStudent>> schoolCollect = paperStudents.stream().collect(Collectors.groupingBy(MsrStudent::getStudentid));
        for (Map.Entry<Integer, List<MsrStudent>> entry : schoolCollect.entrySet()) {
            Integer studentid = entry.getKey();
            MsrStudent cs = new MsrStudent();
            List<MsrStudent> list = entry.getValue();
            cs.setClassid(0);
            cs.setStudentid(studentid);
            cs.setStuscore(list.stream().mapToDouble(MsrStudent::getStuscore).sum());
            cs.setSstate(list.stream().max(Comparator.comparing(MsrStudent::getSstate)).get().getSstate());
            cs.setScorerate(N_Utils.getDoubleDivideAndMulitiply(cs.getStuscore(), fullscore));
            schoolStudents.add(cs);
        }
        List<MsrStudent> paperStudents2 = new ArrayList<>();
        Map<String, List<MsClassStudent>> sClassStudent = students.stream().collect(Collectors.groupingBy(s -> s.getSubjectid() + "_" + s.getStudentid()));
        for (MsrStudent paperStudent : paperStudents) {
            String subjectid = paperStudent.getSubjectid();
            Integer studentid = paperStudent.getStudentid();
            Double stuscore = paperStudent.getStuscore();
            Double fullscore1 = paperStudent.getFullscore();
            String key = subjectid + "_" + studentid;
            for (MsClassStudent student : sClassStudent.get(key)) {
                MsrStudent cs = new MsrStudent();
                cs.setExamid(examid);
                cs.setClassid(student.getClassid());
                cs.setStudentid(student.getStudentid());
                cs.setSubjectid(subjectid);
                cs.setStuscore(stuscore);
                cs.setFullscore(fullscore1);
                paperStudents2.add(cs);
            }
        }
        String subjectid;
        Map<String, Double> bjdfMap = new HashMap<>();//班级得分
        Map<Integer, List<MsrStudent>> cCollect = paperStudents2.stream().collect(Collectors.groupingBy(MsrStudent::getClassid));
        for (Map.Entry<Integer, List<MsrStudent>> entry : cCollect.entrySet()) {
            Integer classid = entry.getKey();
            subjectid = "zf";
            List<MsrStudent> studentList = entry.getValue();

            Map<Integer, List<MsrStudent>> sCollect = studentList.stream().collect(Collectors.groupingBy(MsrStudent::getStudentid));
            List<MsrStudent> rstudents = new ArrayList<>();
            for (Map.Entry<Integer, List<MsrStudent>> csEntry : sCollect.entrySet()) {
                MsrStudent cs = new MsrStudent();
                cs.setExamid(examid);
                cs.setClassid(classid);
                cs.setStudentid(csEntry.getKey());
                cs.setSubjectid(subjectid);
                cs.setStuscore(csEntry.getValue().stream().mapToDouble(MsrStudent::getStuscore).sum());
                cs.setFullscore(fullscore);
                rstudents.add(cs);
            }
            bjdfMap.put(classid + "_" + subjectid, rstudents.stream().mapToDouble(MsrStudent::getStuscore).sum());
            setRank(rstudents, classStuRank, 3);//班级学生分数排名
            classStudents.addAll(rstudents);
            Map<String, List<MsrStudent>> subCollect = studentList.stream().collect(Collectors.groupingBy(MsrStudent::getSubjectid));
            for (Map.Entry<String, List<MsrStudent>> ssEntry : subCollect.entrySet()) {
                subjectid = ssEntry.getKey();
                rstudents = ssEntry.getValue();
                bjdfMap.put(classid + "_" + subjectid, rstudents.stream().mapToDouble(MsrStudent::getStuscore).sum());
                setRank(rstudents, classSubStuRank, 4);//班级科目学生分数排名
                classSubjectStudents.addAll(rstudents);
            }
        }
        setRank(schoolStudents, schoolStuRank, 1);//学校学生分数排名

        List<MsrStudent> schoolStudents2 = schoolStudents.stream().filter(s -> s.getSstate().equals(2)).collect(Collectors.toList());//参与统计学生列表
        DoubleSummaryStatistics schoolStatistics = schoolStudents2.stream().mapToDouble(MsrStudent::getStuscore).summaryStatistics();
        double schoolStatisticsMax = N_Utils.handleInfinity(schoolStatistics.getMax());
        //最低分均不包括0分
        double schoolStatisticsMin = N_Utils.handleInfinity(schoolStatistics.getMin());
        double schoolStatisticsAverage = N_Utils.formatDouble(schoolStatistics.getAverage(), 2);

        //总体分析
        MsrExam msrExam = new MsrExam();
        msrExam.setExamid(examid);
        msrExam.setClassnum(classnum);
        msrExam.setTotalnum(totalnum);
        msrExam.setStunum(stunum);
        msrExam.setMissnum(missnum);
        msrExam.setSubjectnum(subjects.size());
        msrExam.setMaxscore(schoolStatisticsMax);
        msrExam.setMinscore(schoolStatisticsMin);
        msrExam.setCreatetime(N_Utils.getSecondTimestamp());
        msrExamMapper.insert(msrExam);

        //科目分析
        Map<String, MsrSubject> subjectMap = new HashMap<>();//科目分析map
        Map<String, List<MsrStudent>> subjectCollect = paperStudents.stream().collect(Collectors.groupingBy(MsrStudent::getSubjectid));
        List<MsrSubject> msrSubjects = new ArrayList<>();
        MsrSubject msrSubject = new MsrSubject();
        msrSubject.setExamid(examid);
        subjectid = "zf";
        msrSubject.setSubjectid(subjectid);
        msrSubject.setSubjectname("总分");
        msrSubject.setFullscore(fullscore);
        msrSubject.setMaxscore(schoolStatisticsMax);
        msrSubject.setMinscore(schoolStatisticsMin);
        msrSubject.setAvgscore(schoolStatisticsAverage);
        msrSubject.setTotalnum(totalnum);
        msrSubject.setStunum(stunum);
        msrSubject.setMissnum(missnum);
        setSubject(msrSubject, schoolStudents2);
        msrSubjects.add(msrSubject);
        subjectMap.put(subjectid, msrSubject);
        for (MsPaper subject : subjects) {
            subjectid = subject.getSubjectid();
            msrSubject = new MsrSubject();
            msrSubject.setExamid(examid);
            msrSubject.setSubjectid(subjectid);
            msrSubject.setSubjectname(subject.getSubjectname());
            msrSubject.setFullscore(subject.getPscore());
            List<MsrStudent> studentList = subjectCollect.get(subjectid);
            setRank(studentList, schoolSubStuRank, 2);//学校科目学生分数排名
            List<MsrStudent> studentList2 = studentList.stream().filter(s -> s.getSstate().equals(2)).collect(Collectors.toList());//参与统计学生列表
            totalnum = studentList.size();
            stunum = studentList2.size();
            missnum = totalnum - stunum;
            msrSubject.setTotalnum(totalnum);
            msrSubject.setStunum(stunum);
            msrSubject.setMissnum(missnum);
            DoubleSummaryStatistics subjectStatistics = studentList2.stream().mapToDouble(MsrStudent::getStuscore).summaryStatistics();
            double subjectStatisticsMax = N_Utils.handleInfinity(subjectStatistics.getMax());
            //最低分均不包括0分
            double subjectStatisticsMin = N_Utils.handleInfinity(subjectStatistics.getMin());
            double subjectStatisticsAverage = N_Utils.formatDouble(subjectStatistics.getAverage(), 2);
            msrSubject.setMaxscore(subjectStatisticsMax);
            msrSubject.setMinscore(subjectStatisticsMin);
            msrSubject.setAvgscore(subjectStatisticsAverage);
            setSubject(msrSubject, studentList2);
            msrSubjects.add(msrSubject);
            subjectMap.put(subjectid, msrSubject);
        }
        List<Double> avgScores = msrSubjects.stream().map(MsrSubject::getAvgscore).sorted().collect(Collectors.toList());
        Map<Double, Integer> fspm = MarkingUtil.fspm(avgScores);//平均分排名
        for (MsrSubject subject : msrSubjects) {
            subject.setAvgrank(fspm.get(subject.getAvgscore()));
            subject.setAvgrankrate(MarkingUtil.div(subject.getAvgrank(), subject.getTotalnum()));
        }
        msrSubjectMapper.insertList(msrSubjects);

        //学生成绩分析
        for (MsrStudent student : classStudents) {
            setStudent(student, classStuRank, schoolStuRank, classSubStuRank, schoolSubStuRank, bjdfMap, subjectMap, djList);
        }
        for (MsrStudent student : classSubjectStudents) {
            setStudent(student, classStuRank, schoolStuRank, classSubStuRank, schoolSubStuRank, bjdfMap, subjectMap, djList);
        }
        List<MsrStudent> msrStudents = new ArrayList<>();
        msrStudents.addAll(classStudents);
        msrStudents.addAll(classSubjectStudents);
        if (N_Utils.isListNotEmpty(msrStudents)) msrStudentMapper.insertList(msrStudents);

        //科目班级分数分布区间分析、班级学科分析
        //区间参数
        Map<String, List<SzJsonVo>> qjMap = getReportParams(examid, "scoresection", "xk");
        List<MsrSubjectClass> msrSubjectClasses = new ArrayList<>();//科目班级分数分布区间分析
        List<MsrClassSubject> msrClassSubjects = new ArrayList<>();//班级学科分析
        List<MsrClassTop> msrClassTops = new ArrayList<>();//班级学科前60分析
        Map<String, ClassTopVo> lsMap = new HashMap<>();//处于年级60%得分map
        for (MsrSubject subject : msrSubjects) {
            Double lsscore = subject.getLsscore();
            String subjectid1 = subject.getSubjectid();
            List<MsrStudent> lsstudents = msrStudents.stream().filter(s -> s.getSubjectid().equals(subjectid1) && s.getStuscore().compareTo(lsscore) >= 0).collect(Collectors.toList());
            ClassTopVo classTopVo = new ClassTopVo();
            classTopVo.setSubjectid(subjectid1);
            classTopVo.setLsscore(lsscore);
            classTopVo.setXtnum(lsstudents.size());
            double average = lsstudents.stream().mapToDouble(MsrStudent::getStuscore).summaryStatistics().getAverage();
            classTopVo.setXtavgscore(N_Utils.formatDouble(average, 2));
            String xtstuids = lsstudents.stream().map(s -> s.getStudentid().toString()).collect(Collectors.joining(","));
            classTopVo.setXtstuids(xtstuids);
            lsMap.put(subjectid1, classTopVo);
        }
        Map<String, List<MsrStudent>> scCollect = msrStudents.stream().collect(Collectors.groupingBy(s -> s.getClassid() + "_" + s.getSubjectid()));
        for (Map.Entry<String, List<MsrStudent>> entry : scCollect.entrySet()) {
            String key = entry.getKey();
            Integer classid = N_Utils.obj2Int(key.split("_")[0]);
            subjectid = key.split("_")[1];
            List<SzJsonVo> list = qjMap.get(subjectid);
            List<MsrStudent> studentList = entry.getValue();
            totalnum = studentList.size();//区间总人数
            Double fullscore1 = studentList.get(0).getFullscore();
            List<RankGroupVo> fdScore = MarkingUtil.fdScore(fullscore1, N_Utils.obj2Int(list.get(0).getDjvalue()));
            for (RankGroupVo rg : fdScore) {
                int minscore = rg.getMinvalue();
                int maxscore = rg.getMaxvalue();
                String qjname = rg.getGroupname();
                double min = MarkingUtil.objToDouble(minscore);
                double max = MarkingUtil.objToDouble(maxscore);
                List<MsrStudent> qjstudents;
                if (fullscore1.compareTo(max) == 0) {
                    qjstudents = studentList.stream().filter(s -> s.getStuscore().compareTo(min) >= 0).collect(Collectors.toList());
                } else {
                    qjstudents = studentList.stream().filter(s -> s.getStuscore().compareTo(min) >= 0 && s.getStuscore().compareTo(max) < 0).collect(Collectors.toList());
                }
                int qjnum = qjstudents.size();
                MsrSubjectClass msrSubjectClass = new MsrSubjectClass();
                msrSubjectClass.setExamid(examid);
                msrSubjectClass.setSubjectid(subjectid);
                msrSubjectClass.setClassid(classid);
                msrSubjectClass.setQjname(qjname);
                msrSubjectClass.setQjnum(qjnum);
                msrSubjectClass.setQjrate(N_Utils.getIntegerDivideAndMulitiply(qjnum, totalnum));
                double qjavg = qjstudents.stream().mapToDouble(MsrStudent::getStuscore).summaryStatistics().getAverage();
                msrSubjectClass.setQjavgrate(N_Utils.getDoubleDivideAndMulitiply(qjavg, fullscore1));
                String stuids = qjstudents.stream().map(s -> s.getStudentid().toString()).collect(Collectors.joining(","));
                msrSubjectClass.setStuids(stuids);
                msrSubjectClasses.add(msrSubjectClass);
            }

            MsrClassSubject msrClassSubject = new MsrClassSubject();
            msrClassSubject.setExamid(examid);
            msrClassSubject.setSubjectid(subjectid);
            msrClassSubject.setClassid(classid);
            msrClassSubject.setFullscore(fullscore1);
            msrClassSubjects.add(msrClassSubject);

            MsrClassTop msrClassTop = new MsrClassTop();
            msrClassTop.setExamid(examid);
            msrClassTop.setClassid(classid);
            msrClassTop.setSubjectid(subjectid);
            msrClassTop.setStunum(totalnum);
            ClassTopVo classTopVo = lsMap.get(subjectid);//处于年级60%得分
            Double lsscore = classTopVo.getLsscore();
            List<MsrStudent> lsstudents = studentList.stream().filter(s -> s.getStuscore().compareTo(lsscore) >= 0).collect(Collectors.toList());
            msrClassTop.setBtnum(lsstudents.size());
            msrClassTop.setBtrate(N_Utils.getIntegerDivideAndMulitiply(msrClassTop.getBtnum(), totalnum));
            double average = lsstudents.stream().mapToDouble(MsrStudent::getStuscore).summaryStatistics().getAverage();
            msrClassTop.setBtavgscore(N_Utils.formatDouble(average, 2));
            String btstuids = lsstudents.stream().map(s -> s.getStudentid().toString()).collect(Collectors.joining(","));
            msrClassTop.setBtstuids(btstuids);
            msrClassTop.setXtnum(classTopVo.getXtnum());
            msrClassTop.setXtavgscore(classTopVo.getXtavgscore());
            msrClassTop.setXtstuids(classTopVo.getXtstuids());
            msrClassTops.add(msrClassTop);
        }
        msrSubjectClassMapper.insertList(msrSubjectClasses);
        msrClassSubjectMapper.insertList(msrClassSubjects);
        msrClassTopMapper.insertList(msrClassTops);

        saveClassSubjectGrade(examid, "scorerate", "zh", msrStudents);//综合报告-班级学科四率等级分析
        saveClassSubjectGrade(examid, "scorerate", "xj", msrStudents);//校级报告-班级学科四率等级分析

        examMapper.updateExamState(examid, 3);
    }

    //班级学科四率等级分析
    private void saveClassSubjectGrade(Integer examid, String rpbelong, String reportcode, List<MsrStudent> msrStudents) {
        Map<String, List<SzJsonVo>> qjMap = getReportParams(examid, rpbelong, reportcode);
        List<MsrClassSubjectGrade> msrClassSubjectGrades = new ArrayList<>();//班级学科分析
        String subjectid;
        int totalnum;
        Map<String, List<MsrStudent>> scCollect = msrStudents.stream().collect(Collectors.groupingBy(s -> s.getClassid() + "_" + s.getSubjectid()));
        List<Integer> nums = new ArrayList<>();
        for (Map.Entry<String, List<MsrStudent>> entry : scCollect.entrySet()) {
            String key = entry.getKey();
            Integer classid = N_Utils.obj2Int(key.split("_")[0]);
            subjectid = key.split("_")[1];
            List<SzJsonVo> list = qjMap.get(subjectid);
            List<MsrStudent> studentList = entry.getValue();
            totalnum = studentList.size();//区间总人数
            Double fullscore1 = studentList.get(0).getFullscore();
            List<RankGroupVo> fdScore = MarkingUtil.fdScore(fullscore1, N_Utils.obj2Int(list.get(0).getDjvalue()));
            for (RankGroupVo rg : fdScore) {
                int minscore = rg.getMinvalue();
                int maxscore = rg.getMaxvalue();
                String qjname = rg.getGroupname();
                double min = MarkingUtil.objToDouble(minscore);
                double max = MarkingUtil.objToDouble(maxscore);
                List<MsrStudent> qjstudents;
                if (fullscore1.compareTo(max) == 0) {
                    qjstudents = studentList.stream().filter(s -> s.getStuscore().compareTo(min) >= 0).collect(Collectors.toList());
                } else {
                    qjstudents = studentList.stream().filter(s -> s.getStuscore().compareTo(min) >= 0 && s.getStuscore().compareTo(max) < 0).collect(Collectors.toList());
                }
                int qjnum = qjstudents.size();
                MsrClassSubjectGrade classSubjectGrade = new MsrClassSubjectGrade();
                classSubjectGrade.setExamid(examid);
                classSubjectGrade.setSubjectid(subjectid);
                classSubjectGrade.setClassid(classid);
                classSubjectGrade.setReportcode(reportcode);
                classSubjectGrade.setRange(qjname);
                classSubjectGrade.setRangenum(qjnum);
                classSubjectGrade.setRangerate(N_Utils.getIntegerDivideAndMulitiply(qjnum, totalnum));
                //double qjavg = qjstudents.stream().mapToDouble(MsrStudent::getStuscore).summaryStatistics().getAverage();
                //classSubjectGrade.setQjavgrate(N_Utils.getDoubleDivideAndMulitiply(qjavg, fullscore1));
                String stuids = qjstudents.stream().map(s -> s.getStudentid().toString()).collect(Collectors.joining(","));
                classSubjectGrade.setStuids(stuids);
                msrClassSubjectGrades.add(classSubjectGrade);
                nums.add(qjnum);
            }
        }
        Map<Integer, Integer> rspm = MarkingUtil.rspm(nums);//人数排名
        for (MsrClassSubjectGrade classSubjectGrade : msrClassSubjectGrades) {
            int rangenum = classSubjectGrade.getRangenum();
            int rank = rspm.get(rangenum);
            classSubjectGrade.setRangenumrank(rank);
        }
        msrClassSubjectGradeMapper.insertList(msrClassSubjectGrades);
    }

    //获取报告参数
    private Map<String, List<SzJsonVo>> getReportParams(Integer examid, String rpbelong, String reportcode) {
        List<MsrReportparam> list = msrReportparamMapper.listReportparamByRpbelong(examid, rpbelong, reportcode);
        if (N_Utils.isListEmpty(list)) {
            list = msrReportparamMapper.listReportparamByRpbelong(examid, rpbelong, "all");
        }
        Map<String, List<SzJsonVo>> qjMap = new HashMap<>();
        for (MsrReportparam param : list) {
            qjMap.put(param.getSubjectid(), JSON.parseArray(param.getSzjson(), SzJsonVo.class));
        }
        return qjMap;
    }

    //设置科目分析计算参数
    private void setSubject(MsrSubject msrSubject, List<MsrStudent> schoolStudents2) {
        Double avgscore = msrSubject.getAvgscore();
        Double fullscore = msrSubject.getFullscore();
        msrSubject.setScorerate(N_Utils.getDoubleDivideAndMulitiply(avgscore, fullscore));
        List<Double> scores = schoolStudents2.stream().map(MsrStudent::getStuscore).sorted().collect(Collectors.toList());
        double bzc = MarkingUtil.bzc(scores);
        msrSubject.setBzc(bzc);
        msrSubject.setCyxs(N_Utils.getDoubleDivideAndMulitiply(bzc, avgscore));
        msrSubject.setNd(MarkingUtil.div(avgscore, fullscore));
        msrSubject.setQfd(MarkingUtil.qfd(scores));
        List<Double> scoreRates = schoolStudents2.stream().map(MsrStudent::getScorerate).sorted().collect(Collectors.toList());
        double dfllc = MarkingUtil.sub(scoreRates.get(scoreRates.size() - 1), scoreRates.get(0));
        msrSubject.setDfllc(dfllc);
        msrSubject.setDflxdlc(MarkingUtil.div(dfllc, avgscore));
        msrSubject.setEsscore(MarkingUtil.cynjdf(scores, 0.2));
        msrSubject.setSsscore(MarkingUtil.cynjdf(scores, 0.4));
        msrSubject.setLsscore(MarkingUtil.cynjdf(scores, 0.6));
        msrSubject.setBsscore(MarkingUtil.cynjdf(scores, 0.8));
    }

    //设置学生分析计算参数
    private void setStudent(MsrStudent student, Map<String, StuRankVo> classStuRank, Map<String, StuRankVo> schoolStuRank
            , Map<String, StuRankVo> classSubStuRank, Map<String, StuRankVo> schoolSubStuRank
            , Map<String, Double> bjdfMap, Map<String, MsrSubject> subjectMap, List<SzJsonVo> djList) {
        String subjectid;
        Integer classid = student.getClassid();
        Integer studentid = student.getStudentid();
        subjectid = student.getSubjectid();
        MsrSubject msrSubject = subjectMap.get(subjectid);
        Double pjf = msrSubject.getAvgscore();//年级平均分
        student.setBzf(MarkingUtil.bzf(bjdfMap.get(classid + "_" + subjectid), pjf, msrSubject.getBzc()));
        StuRankVo classRankVo;
        StuRankVo schoolRankVo;
        if (subjectid.equals("zf")) {
            classRankVo = classStuRank.get(classid + "_" + studentid);
            schoolRankVo = schoolStuRank.get(studentid.toString());
        } else {
            classRankVo = classSubStuRank.get(classid + "_" + subjectid + "_" + studentid);
            schoolRankVo = schoolSubStuRank.get(subjectid + "_" + studentid);
        }
        Integer classrank = classRankVo.getRank();
        Integer schoolrank = schoolRankVo.getRank();
        student.setCjl(MarkingUtil.cjl(student.getStuscore(), pjf));
        student.setClassrank(classrank);
        student.setSchoolrank(schoolrank);
        double rankrate = MarkingUtil.div(schoolrank, msrSubject.getTotalnum());
        student.setRankrate(rankrate);//个人成绩排名率=自己所在学科成绩分数的名次/总人数
        student.setXkqrz(MarkingUtil.sub(msrSubject.getAvgrank(), rankrate));//学科强弱值 = 平均成绩排名率- 个人成绩排名率
        double dfv = N_Utils.getDoubleDivideAndMulitiply(student.getStuscore(), msrSubject.getFullscore());
        student.setGradevalue(MarkingUtil.dj(dfv, djList));
        student.setHighschool(N_Utils.getIntegerDivideAndMulitiply(schoolrank, schoolRankVo.getNum2()));
    }

    //分数排名
    private void setRank(List<MsrStudent> students, Map<String, StuRankVo> rank, Integer code) {
        List<Map.Entry<Double, List<MsrStudent>>> gslist2 = students.stream()
                                                                    .collect(Collectors.groupingBy(MsrStudent::getStuscore)).entrySet()
                                                                    .stream().sorted((s1, s2) -> -Double.compare(s1.getKey(), s2.getKey())).collect(Collectors.toList());
        int index = 1;
        StuRankVo rankVo;
        String key;
        List<StuRankVo> rankVos = new ArrayList<>();
        for (Map.Entry<Double, List<MsrStudent>> entry : gslist2) {
            int num = entry.getValue().size();
            for (MsrStudent cgs : entry.getValue()) {
                Integer studentid = cgs.getStudentid();
                Integer classid = cgs.getClassid();
                String subjectid = cgs.getSubjectid();
                if (code == 1) {//学校学生
                    key = studentid.toString();
                } else if (code == 2) {//学校科目学生
                    key = subjectid + "_" + studentid;
                } else if (code == 3) {//班级学生
                    key = classid + "_" + studentid;
                } else {//班级科目学生
                    key = classid + "_" + subjectid + "_" + studentid;
                }
                rankVo = new StuRankVo();
                rankVo.setKey(key);
                rankVo.setRank(index);
                rankVo.setNum(num);
                rankVos.add(rankVo);
                rank.put(key, rankVo);
            }
            index = index + num;
        }
        List<Integer> codes = Arrays.asList(1, 2);//计算超过当前排名人数
        if (!codes.contains(code)) return;
        for (StuRankVo vo : rankVos) {
            Integer rank1 = vo.getRank();
            int sum = rankVos.stream().filter(r -> rank1.compareTo(r.getRank()) < 0).mapToInt(StuRankVo::getNum).sum();
            vo.setNum2(sum);
        }
    }
}
